#######################################################################
# Copyright (c) 2017 Josef Spidlen, Ph.D., FlowJo, LLC
#
# License
# The software is distributed under the terms of the 
# Artistic License 2.0
# http://www.r-project.org/Licenses/Artistic-2.0
# 
# Disclaimer
# This software and documentation come with no warranties of any kind.
# This software is provided "as is" and any express or implied 
# warranties, including, but not limited to, the implied warranties of
# merchantability and fitness for a particular purpose are disclaimed.
# In no event shall the  copyright holder be liable for any direct, 
# indirect, incidental, special, exemplary, or consequential damages
# (including but not limited to, procurement of substitute goods or 
# services; loss of use, data or profits; or business interruption)
# however caused and on any theory of liability, whether in contract,
# strict liability, or tort arising in any way out of the use of this 
# software.
######################################################################

suppressMessages(library("scImpute")) # Tested with scImpute_0.0.4
suppressMessages(library("parallel")) # This is a base lib, used to detect number of cores available
sessionInfo()

# cat("FJ-DATA-FILE-PATH         = FJ_DATA_FILE_PATH \n")
# cat("FJ-CSV-OUPUT-FILE         = FJ_CSV_OUPUT_FILE \n")
# cat("FJ-PAR-K                  = FJ_PAR_K \n")
# cat("FJ-PAR-CORES              = FJ_PAR_CORES \n")
# cat("FJ-PAR-DROPTH             = FJ_PAR_DROPTH \n")
# cat("FJ-PARAMS-LIST            = FJ-PARAMS-LIST \n")     ## This is too long, we don't want this... that's why "-" instead of "_" 

outCSVPath <- "FJ_CSV_OUPUT_FILE"
inCSVPath <- "FJ_DATA_FILE_PATH"

# scimpute options
ncores <- FJ_PAR_CORES
if (ncores <= 0) {
	ncores <- detectCores()
}

# Both Mac and Unix/Linux have .Platform$OS.type == "unix"; this is a test for Windows, this package supports only 1 core on Windows
if (.Platform$OS.type != "unix")
	ncores <- 1
	
ncores
Kcluster <- FJ_PAR_K
drop_thre = FJ_PAR_DROPTH

originalDir <- getwd()

setwd(dirname(outCSVPath))
subDir <- paste0(tools::file_path_sans_ext(basename(outCSVPath)), ".R")

if (file.exists(subDir)) {
	setwd(file.path(getwd(), subDir))
} else {
	dir.create(file.path(getwd(), subDir))
	setwd(file.path(getwd(), subDir))
}

scimpute(
  count_path = inCSVPath, 
  infile = "csv",
  outfile = "csv",
  out_dir = "./",
  labeled = FALSE,
  drop_thre = drop_thre,
  Kcluster = Kcluster,
  ncores = ncores)

file.rename(file.path(getwd(), "scimpute_count.csv"), outCSVPath)


setwd(originalDir)

# R seems to be saving the .RData when exiting, so let's clean up to at least make that tiny (i.e., empty environment)
rm(list=ls())
