package com.flowjo.plugin.scimpute;

import java.io.BufferedReader;
import java.io.File;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.StringWriter;
import java.util.List;

import com.treestar.flowjo.engine.EngineManager;
import com.treestar.flowjo.engine.utility.RFlowCalculator;
import com.treestar.lib.file.FileUtil;

public class SCImputeCalc extends RFlowCalculator {
	
	private final static String scriptTemplatePath = "r/RScript.scImpute.Template.R";
	
	public File runCalc(File sampleFile, String sampleName, List<String> parameterNames, int k, int cores, double dropThre, String outputFolderPath)
    {
		sampleName = sampleName.replaceAll(".ExtNode", "").replaceAll(".fcs", "").replaceAll(".LMD", "").trim();
        File outputFolder = new File(outputFolderPath);
        StringWriter scriptWriter = new StringWriter();
        File rScriptOutput = createScript(sampleFile, sampleName, parameterNames, k, cores, dropThre, outputFolder, scriptWriter);
        if(rScriptOutput == null) return null;        
        String scriptFileName = (new StringBuilder()).append("RScript.scImpute.").append(System.currentTimeMillis()).append(".R").toString().replaceAll(" ", "_");
        try
        {
            fScriptFile = new File(outputFolderPath, scriptFileName);
            FileUtil.write(fScriptFile, scriptWriter.toString());
            executeRBatch(scriptFileName);
        }
        catch(Exception e) 
        {
            e.printStackTrace();
        }
        return rScriptOutput;
    }

	private File createScript(File sampleFile, String sampleName, List<String> parameterNames, int k, int cores, double dropThre, File outputFolder, StringWriter scriptWriter)
	{
        InputStream scriptStream = SCImputeCalc.class.getResourceAsStream(scriptTemplatePath);
        
        String outFileName = (new StringBuilder()).append(fixFileName(sampleName)).append(".scImpute").append(".csv").toString();
        if(outputFolder == null) outputFolder = sampleFile.getParentFile();
        
        File outFile = new File(outputFolder, outFileName);
        outFileName = outFile.getAbsolutePath();
        String dataFilePath = sampleFile.getAbsolutePath();

        if(EngineManager.isWindows()) outFileName = outFileName.replaceAll("\\\\", "/");
        if(EngineManager.isWindows()) dataFilePath = dataFilePath.replaceAll("\\\\", "/");

        BufferedReader rTemplateReader = null;
        try {
			rTemplateReader = new BufferedReader(new InputStreamReader(scriptStream));
		} catch (Exception e) {
			e.printStackTrace();
			return null;
		}
        
        String scriptLine;
        try {
			while((scriptLine = rTemplateReader.readLine()) != null) 
			{
			    scriptLine = scriptLine.replaceAll("FJ_DATA_FILE_PATH", dataFilePath);
			    scriptLine = scriptLine.replaceAll("FJ_CSV_OUPUT_FILE", outFileName);
			    scriptLine = scriptLine.replaceAll("FJ_PAR_K", "" + k);
			    scriptLine = scriptLine.replaceAll("FJ_PAR_CORES", "" + cores);
			    scriptLine = scriptLine.replaceAll("FJ_PAR_DROPTH", "" + dropThre);
			    
			    if (scriptLine.contains("FJ_PARAMS_LIST")) {
			    	String parListStr = String.join("\",\"", parameterNames);
			    	scriptLine = scriptLine.replaceAll("FJ_PARAMS_LIST", "\"" + parListStr + "\"");
			    }
			    scriptWriter.append(scriptLine).append('\n');
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
        
        if(rTemplateReader != null) {
        	try { rTemplateReader.close(); }
        	catch (Exception e) { e.printStackTrace(); }
        }
        
        return outFile;
	}

	
	public static String fixFileName(String fileName) {
		return fileName.
			replaceAll(" ", "_").replaceAll("/", "_").replaceAll("\\\\", "_").
	        replaceAll("\\:", "_").replaceAll("\\*", "_").replaceAll("\\?", "_").
	        replaceAll("\"", "_").replaceAll("\\|", "_");
	}
}
